@@ -258,7 +258,7 @@ static inline void grubenv_close(struct grubenv_t *grubenv)
 /* I feel that '#' and '=' characters should be forbidden. Although it's not
  * explicitly mentioned in original grub env code, they may cause unexpected
  * behavior */
-int bootloader_env_set(const char *name, const char *value)
+static int do_env_set(const char *name, const char *value)
 {
 	static struct grubenv_t grubenv;
 	int ret;
@@ -280,7 +280,7 @@ cleanup:
 	return ret;
 }
 
-int bootloader_env_unset(const char *name)
+static int do_env_unset(const char *name)
 {
 	static struct grubenv_t grubenv;
 	int ret = 0;
@@ -301,7 +301,7 @@ cleanup:
 	return ret;
 }
 
-char *bootloader_env_get(const char *name)
+static char *do_env_get(const char *name)
 {
 	static struct grubenv_t grubenv;
 	char *value = NULL, *var;
@@ -322,7 +322,7 @@ cleanup:
 
 }
 
-int bootloader_apply_list(const char *script)
+static int do_apply_list(const char *script)
 {
 	static struct grubenv_t grubenv;
 	int ret = 0;
@@ -343,3 +343,16 @@ cleanup:
 	grubenv_close(&grubenv);
 	return ret;
 }
+
+static bootloader grub = {
+	.env_get = &do_env_get,
+	.env_set = &do_env_set,
+	.env_unset = &do_env_unset,
+	.apply_list = &do_apply_list
+};
+
+__attribute__((constructor))
+static void grub_probe(void)
+{
+	(void)register_bootloader("grub", &grub);
+}
