@@ -439,7 +439,9 @@ static int channel_callback_xferinfo(void *p, curl_off_t dltotal, curl_off_t dln
 	else
 		data->percent = percent;
 
-	DEBUG("Downloaded %d%% (%lu of %lu kB).", percent, dlnow / 1024, dltotal / 1024);
+	DEBUG("Downloaded %d%% (%zu of %zu kB).", percent,
+		(size_t)dlnow / 1024,
+		(size_t)dltotal / 1024);
 	swupdate_download_update(percent, dltotal);
 
 	return 0;
@@ -1025,6 +1027,8 @@ static channel_op_res_t channel_put_method(channel_t *this, void *data)
 	channel_op_res_t result = CHANNEL_OK;
 	channel_data_t *channel_data = (channel_data_t *)data;
 	channel_data->offs = 0;
+	output_data_t outdata = {};
+	write_callback_t wrdata = { .this = this, .channel_data = channel_data, .outdata = &outdata };
 
 	if ((result = channel_set_content_type(this, channel_data)) !=
 	    CHANNEL_OK) {
@@ -1037,6 +1041,10 @@ static channel_op_res_t channel_put_method(channel_t *this, void *data)
 		goto cleanup_header;
 	}
 
+	if ((result = setup_reply_buffer(channel_curl->handle, &wrdata)) != CHANNEL_OK) {
+		goto cleanup_header;
+	}
+
 	if ((curl_easy_setopt(channel_curl->handle, CURLOPT_PUT, 1L) != CURLE_OK) ||
 	    (curl_easy_setopt(channel_curl->handle, CURLOPT_UPLOAD, 1L) != CURLE_OK)) {
 		ERROR("Set PUT channel method option failed.");
@@ -1072,6 +1080,7 @@ static channel_op_res_t channel_put_method(channel_t *this, void *data)
 	channel_log_reply(result, channel_data, NULL);
 
 cleanup_header:
+	outdata.memory != NULL ? free(outdata.memory) : (void)0;
 	curl_easy_reset(channel_curl->handle);
 	curl_slist_free_all(channel_curl->header);
 	channel_curl->header = NULL;
