@@ -485,8 +485,6 @@ static int mkfs_prepare(const char *device_name, struct ext2_super_block *pfs_pa
 	use_bsize = sys_page_size;
 	if (lsector_size && use_bsize < lsector_size)
 		use_bsize = lsector_size;
-	if ((blocksize < 0) && (use_bsize < (-blocksize)))
-		use_bsize = -blocksize;
 	blocksize = use_bsize;
 	fs_blocks_count /= (blocksize / 1024);
 
@@ -619,14 +617,6 @@ static int mkfs_prepare(const char *device_name, struct ext2_super_block *pfs_pa
 			(unsigned long long) fs_blocks_count);
 	}
 
-	if (ext2fs_has_feature_casefold(pfs_param) &&
-	    ext2fs_has_feature_encrypt(pfs_param)) {
-		ERROR("The encrypt and casefold features are not "
-			  "compatible.\nThey can not be both enabled "
-			  "simultaneously.");
-		return -EINVAL;
-	}
-
 	/* Don't allow user to set both metadata_csum and uninit_bg bits. */
 	if (ext2fs_has_feature_metadata_csum(pfs_param) &&
 	    ext2fs_has_feature_gdt_csum(pfs_param))
@@ -735,7 +725,7 @@ static int mkfs_prepare(const char *device_name, struct ext2_super_block *pfs_pa
 		(ext2fs_blocks_count(pfs_param) * blocksize) / inode_ratio;
 
 	if ((((unsigned long long)pfs_param->s_inodes_count) *
-	     (inode_size ? inode_size : EXT2_GOOD_OLD_INODE_SIZE)) >=
+	     inode_size) >=
 	    ((ext2fs_blocks_count(pfs_param)) *
 	     EXT2_BLOCK_SIZE(pfs_param))) {
 		ERROR("inode_size (%u) * inodes_count "
