@@ -23,5 +23,8 @@ typedef enum {
 	CHANNEL_ELOOP,
 	CHANNEL_EBADMSG,
 	CHANNEL_ENOTFOUND,
-	CHANNEL_EREDIRECT
+	CHANNEL_EREDIRECT,
+	CHANNEL_ESSLCERT,
+	CHANNEL_ESSLCONNECT,
+	CHANNEL_REQUEST_PENDING,
 } channel_op_res_t;
